#!/usr/bin/env python
import sys
sys.path.append('dataset_toolkit')

from toc_dataset_toolkit import TOC_Dataset
from toc_dataset_toolkit.utils.logging import set_log_level
from renderer import resize_camera_parameters

import argparse
from pathlib import Path
import subprocess
import logging



def main(dataset_root, scene_name, resize_factor):
    set_log_level('WARNING')
    dataset = TOC_Dataset(root=dataset_root)
    scene_name_list = dataset.load_scene_name_list()
    scene_id = scene_name_list.index(scene_name)
    print('scene', scene_id, scene_name)

    camera_param = dataset.load_camera_param(scene_id)
    camera_param = resize_camera_parameters(camera_param, resize_factor)
    print(
        f'width: {camera_param.intrinsic.width}, '
        f'height: {camera_param.intrinsic.height}, '
        f'f: {camera_param.intrinsic.get_focal_length()}, '
        f'c: {camera_param.intrinsic.get_principal_point()}, '
    )
    fx, fy = camera_param.intrinsic.get_focal_length()
    cx, cy = camera_param.intrinsic.get_principal_point()
    camera_param_str = ','.join([str(i) for i in [fx, fy, cx, cy]])

    output_path = dataset_root / 'scenes' / scene_name / 'camera_pose'
    database_path =  output_path / 'database.db'
    sparse_model_path =  output_path / 'sparse'
    sparse_model_text_path =  dataset_root / 'scenes' / scene_name / 'colmap_output_all_images_2'
    image_path = dataset_root / 'scenes' / scene_name / 'all_images'

    output_path.mkdir(exist_ok=True, parents=True)
    sparse_model_path.mkdir(exist_ok=True, parents=True)
    sparse_model_text_path.mkdir(exist_ok=True, parents=True)

    run_cmd([
        'colmap', 'feature_extractor',
        '--database_path', str(database_path),
        '--image_path', str(image_path),
        '--ImageReader.camera_params', camera_param_str,
        '--ImageReader.single_camera',  str(1),
        '--ImageReader.camera_model', 'PINHOLE',
    ])

    run_cmd([
        'colmap', 'exhaustive_matcher',
        '--database_path', str(database_path),
    ])

    run_cmd([
        'colmap', 'mapper',
        '--database_path', str(database_path),
        '--image_path', str(image_path),
        '--output_path', str(sparse_model_path),
    ])

    run_cmd([
        'colmap', 'model_converter',
        '--input_path', str(sparse_model_path/'0'),
        '--output_path', str(sparse_model_text_path),
        '--output_type', 'TXT',
    ])


def run_cmd(cmd):
    ret = subprocess.call(cmd)
    if ret != 0:
        logging.error('Problem with ', ' '.join(cmd))
        exit(ret)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--dataset-root', type=Path,
        default='/root/3d_dataset'
    )
    parser.add_argument(
        '--scene-name', type=str,
        default='scene2'
    )
    parser.add_argument('--resize-factor', type=int, default=2)
    args = parser.parse_args()
    main(**args.__dict__)
