import copy
from get_mask import get_mask
from unpack import unpack
from xmlhandler import xmlReader
import argparse
from pose import pose_from_pose_vector, pose_list_from_pose_vector_list
import os
import numpy as np
from xmlhandler import getframeposevectorlist
import open3d as o3d
from render_image import render_image
from model import loadmodel
import cv2
parser = argparse.ArgumentParser()

parser.add_argument('--model_dir', default='models',
                    help='ply model files directory path')
parser.add_argument('--tar_dir', default='tars/11-19',
                    help='directory where the tar files locate')
parser.add_argument('--camera', default='realsense',
                    help='cameara type, realsense or kinect')
parser.add_argument('--mask_dir', default='masks',
                    help='the directory where the mask npy files locate')
parser.add_argument('--rendered_image_dir', default='rendered_image',
                    help='where the rendered images locate')
parser.add_argument('--xml_dir', default='annotations',
                    help='output xml file directory')
parser.add_argument('--data_dir', default='scene_105',
                    help='scene directory name')
parser.add_argument('--object_id_list', default='object_id_list.txt',
                    help='object id list text file name')
parser.add_argument('--object_file_name_list', default='object_file_name_list.txt',
                    help='ascii text file name that specifies the filenames of all possible objects')


FLAGS = parser.parse_args()

if FLAGS.camera == 'realsense':
    ALL_DATA_DIR = os.path.join(FLAGS.data_dir, 'data')
    RGB_IMAGE_DIR = os.path.join(FLAGS.data_dir, 'data', 'rgb')
    DEPTH_IMAGE_DIR = os.path.join(FLAGS.data_dir, 'data', 'depth')
    XML_DIR = os.path.join(FLAGS.data_dir, 'data', FLAGS.xml_dir)
    MASK_DIR = os.path.join(FLAGS.data_dir, 'data', FLAGS.mask_dir)
    RENDERED_IMAGE_DIR = os.path.join(FLAGS.data_dir, 'data', FLAGS.rendered_image_dir)
    # CAMERA_POSE_NUMPY_FILE_NAME=os.path.join(FLAGS.data_dir,'data',FLAGS.camera_pose)
    CAMERA_POSE_NUMPY_FILE_NAME = 'realsense_pose.npy'
elif FLAGS.camera == 'kinect':
    ALL_DATA_DIR = os.path.join(FLAGS.data_dir, 'data_kinect')
    RGB_IMAGE_DIR = os.path.join(FLAGS.data_dir, 'data_kinect', 'rgb')
    DEPTH_IMAGE_DIR = os.path.join(FLAGS.data_dir, 'data_kinect', 'depth')
    XML_DIR = os.path.join(FLAGS.data_dir, 'data_kinect', FLAGS.xml_dir)
    MASK_DIR = os.path.join(FLAGS.data_dir, 'data_kinect', FLAGS.mask_dir)
    RENDERED_IMAGE_DIR = os.path.join(FLAGS.data_dir,'data_kinect',FLAGS.rendered_image_dir)
    # CAMERA_POSE_NUMPY_FILE_NAME=os.path.join(FLAGS.data_dir,'data_kinect',FLAGS.camera_pose)
    CAMERA_POSE_NUMPY_FILE_NAME='kinect_pose.npy'


TAR_DIR = FLAGS.tar_dir
OBJECT_ID_LIST_FILE_NAME=os.path.join(FLAGS.data_dir,FLAGS.object_id_list)
OBJECT_FILE_NAME_LIST_FILE_NAME=FLAGS.object_file_name_list
def get_model_list(model_dir,model_name_list):
    """
    Input:
    model_dir:the directory where the models locate
    model_name_list: the list of model name that want to load

    Output:
    A list of the model instance defind in model.py
    """
    model_list=[]
    print('log:loading models')
    for model_name in model_name_list:
        model_list.append(loadmodel(model_dir,model_name))
        # print('log:loaded model %s' % model_name)
    print('log:models loaded')
    return model_list


def get_model_list_light(model_dir,model_name_list,model_id_list):
    """
    Input:
    model_dir:the directory where the models locate
    model_name_list: the list of model name that want to load
    model_id_list: list of model that want to load
    Output:
    A list of the model instance defind in model.py
    """
    model_list=[]
    print('log:loading models')
    for i in range(len(model_name_list)):
        if i in model_id_list:
            model_list.append(loadmodel(model_dir,model_name_list[i]))
        else:
            model_list.append(None)
        # print('log:loaded model %s' % model_name)
    print('log:models loaded')
    return model_list

def get_model_pcd_list(model_dir,model_name_list):
    """
    Input:
    model_dir:the directory where the models locate
    model_name_list: the list of model name that want to load

    Output:
    A list of the point cloud raed by open3d library
    """
    model_pcd_list=[]
    print('log:loading ply models using open3d')
    for model_name in model_name_list:
        model_pcd_list.append(o3d.io.read_point_cloud(os.path.join(model_dir,model_name)))
    print('log:ply models loaded')
    return model_pcd_list

def main():
    """
    Function:
    The main function that generates the mask for a scene.
    """
    object_file_name_list_file = open(OBJECT_FILE_NAME_LIST_FILE_NAME,'r')
    lines = object_file_name_list_file.readlines()
    objectfilenamelist = []
    for line in lines:
        if not (line == '\n'):
            objectfilenamelist.append(line.replace('\n','').replace('\r',''))
    print('log:loaded object file name list:',end='')
    print(objectfilenamelist)

    if not os.path.exists(XML_DIR) or not os.path.exists(OBJECT_ID_LIST_FILE_NAME):
        unpack(scene_name = FLAGS.data_dir,tar_file_directory = TAR_DIR,scene_directory = '',force_move = True)

    object_id_list_file = open(OBJECT_ID_LIST_FILE_NAME,'r')
    lines = object_id_list_file.readlines()
    objectidlist=[]
    for line in lines:
        if not (line == '\n'):
            objectidlist.append(int(line))
    print('log:loaded object id list:',end='')
    print(objectidlist)

    num_rgb_image = len(os.listdir(RGB_IMAGE_DIR))
    num_depth_image = len(os.listdir(DEPTH_IMAGE_DIR))
    assert num_depth_image == num_rgb_image
    frame_number = num_depth_image
    print('log:total frames number:%d' % frame_number)

    frameposevectorlist = getframeposevectorlist(objectidlist,True,frame_number,XML_DIR)
    assert frame_number == len(frameposevectorlist)
    model_pcd_list = get_model_pcd_list(FLAGS.model_dir,objectfilenamelist)
    # model_list = get_model_list(FLAGS.model_dir,objectfilenamelist)
    model_list = get_model_list_light(FLAGS.model_dir,objectfilenamelist,objectidlist)

    if not os.path.exists(MASK_DIR):
        os.mkdir(MASK_DIR)
    if not os.path.exists(RENDERED_IMAGE_DIR):
        os.mkdir(RENDERED_IMAGE_DIR)
    for i in range(frame_number):
        pose_list =pose_list_from_pose_vector_list(frameposevectorlist[i])
        # print('log:pose_list:%s' % pose_list)
        mask = get_mask(model_pcd_list,FLAGS.camera,pose_list)
        np.save(os.path.join(MASK_DIR,'%04d.npy' % i),mask)
        print('log:generated mask for frame %d/%d' % (i+1,frame_number))
        bg_image = cv2.imread(os.path.join(RGB_IMAGE_DIR,'%04d.png' % i))
        assert bg_image is not None
        cv2.imwrite(os.path.join(RENDERED_IMAGE_DIR,'%04d.png' % i),render_image(model_list = model_list,model_pcd_list = model_pcd_list,background_image = bg_image,camera = FLAGS.camera,pose_list = pose_list))

if __name__ == '__main__':
    main()
