# to do list
# solve bugs with pose.npy
import os
import math
import argparse
import numpy as np
import cv2
from model import Model3D,loadmodel,cachemodel
from xmlhandler import xmlWriter,xmlReader,getframeposevectorlist,getposevectorlist
import copy
from trans3d import get_mat, pos_quat_to_pose_4x4
from trans3d import correct_rotation
from renderer import Renderer,draw_model
from renderer import tear_down as tear_down_renderer
from transforms3d.quaternions import mat2quat, quat2mat
from transforms3d.euler import mat2euler
from check_data import lego_icp
from frame_transform import frameTransformer
from pynput import keyboard
from icp import refine_6d_pose
import open3d as o3d
import copy

# number of points to sample from a mesh
SCENE_MESH_SAMPLE_POINTS = 1000000
OBJECT_MESH_SAMPLE_POINTS = 500000

# ICP alignment threshold
ICP_ALIGNMENT_THRESHOLD = 0.05

parser = argparse.ArgumentParser()
## useless
# parser.add_argument('--rgb_image', default='0000.png', help='background image file name')
# parser.add_argument('--rgb_image_dir',default='scene_04/data/rgb',help='background image files folder')
# parser.add_argument('--depth_image_dir',default='scene_04/data/depth',help='backgroud depth image file name')
# parser.add_argument('--depth_image_filename',default='0000.png',help='backgroud depth image file name')
# parser.add_argument('--output_xml_filename',default='scene.xml',help='file name for output xml file')
## useful
parser.add_argument('--model_dir', default='../../models', help='ply model files directory path')
parser.add_argument('--xml_dir',default='annotations',help='output xml file directory')
parser.add_argument('--camera',default='realsense',help='realsense or kinect')
parser.add_argument('--data_dir',default='scene_31',help='datas directory')
parser.add_argument('--scene_ply',default='./full_point_cloud.ply',help='scene point cloud')
parser.add_argument('--num_frame',default=0,type=int,help='frame number for annotating')
parser.add_argument('--extend_frames',default='True',help='True for extending the annotation results to other frames, False for not')
parser.add_argument('--resume',default='True',help='xml file path to resume annotation')
# parser.add_argument('--camera_pose',default='cam_poses_new.npy',help='npy file describing camera poses')
# for realsense use cam_poses_new.npy in the data folder
# for kinect use cam_poses_new_small0.npy in the data_kinect folder
parser.add_argument('--save_xml',default='True',help='True for saving xml, False for not saving xml, when checking the result, this arg should be set as False')
parser.add_argument('--icp',default='False',help='whether to apply icp algorithm to extended frames')
parser.add_argument('--object_list',default='object_list.txt',help='ascii text file name that specifies the ids of objects that appear in the scene')
parser.add_argument('--object_file_name_list',default='../../object_name_list.txt',help='ascii text file name that specifies the filenames of all possible objects')
FLAGS = parser.parse_args()

if FLAGS.save_xml == 'True':
        IS_SAVE_XML = True
elif FLAGS.save_xml == 'False':
        IS_SAVE_XML = False
else:
        raise ValueError('Invalid input for argument "save_xml"\n"save_xml" should be True or False')

if FLAGS.resume == 'True':
        IS_RESUME = True
elif FLAGS.resume == 'False':
        IS_RESUME = False
else:
        raise ValueError('Invalid input for argument "resume"\n"resume" should be True or False')

if FLAGS.extend_frames == 'True':
        IS_EXTEND_FRAMES = True
elif FLAGS.extend_frames == 'False':
        IS_EXTEND_FRAMES = False
else:
        raise ValueError('Invalid input for argument "extend_frames"\n"extend_frames" should be True or False')

if FLAGS.icp == 'True':
        IS_ICP = True
elif FLAGS.icp == 'False':
        IS_ICP = False
else:
        raise ValueError('Invalid input for argument "icp"\n"icp" should be True or False')

if FLAGS.camera == 'realsense':
        ALL_DATA_DIR =  os.path.join(FLAGS.data_dir)
        RGB_IMAGE_DIR = os.path.join(FLAGS.data_dir, 'rgb_undistort')
        DEPTH_IMAGE_DIR = os.path.join(FLAGS.data_dir, 'rgb_undistort')
        XML_DIR = os.path.join(FLAGS.data_dir,FLAGS.xml_dir)
        PLY_DIR = os.path.join(FLAGS.data_dir,'data',FLAGS.scene_ply)
        # CAMERA_POSE_NUMPY_FILE_NAME=os.path.join(FLAGS.data_dir,'data',FLAGS.camera_pose)
        CAMERA_POSE_NUMPY_FILE_NAME= os.path.join(ALL_DATA_DIR, 'camera_poses.npy')
elif FLAGS.camera == 'kinect':
        ALL_DATA_DIR = os.path.join(FLAGS.data_dir, 'data_kinect')
        RGB_IMAGE_DIR = os.path.join(FLAGS.data_dir,'data_kinect','rgb')
        DEPTH_IMAGE_DIR = os.path.join(FLAGS.data_dir,'data_kinect','depth')
        XML_DIR = os.path.join(FLAGS.data_dir,'data_kinect',FLAGS.xml_dir)
        PLY_DIR = os.path.join(FLAGS.data_dir, 'data_kinect', FLAGS.scene_ply)
        # CAMERA_POSE_NUMPY_FILE_NAME=os.path.join(FLAGS.data_dir,'data_kinect',FLAGS.camera_pose)
        CAMERA_POSE_NUMPY_FILE_NAME='kinect_pose.npy'
else:
        raise ValueError('Invalid input for argument "camera"\n"camera" should be realsense or kinect')


DATA_DIR=FLAGS.data_dir
MODEL_DIR=os.path.join(FLAGS.data_dir,FLAGS.model_dir)
num_frame=FLAGS.num_frame

OBJECT_ID_LIST_FILE_NAME=os.path.join(FLAGS.data_dir,FLAGS.object_list)
OBJECT_FILE_NAME_LIST_FILE_NAME=os.path.join(FLAGS.data_dir,FLAGS.object_file_name_list)

# global variables
moving_speed = 5
object_pose = {
    'pose': np.eye(4),
    'is_new': True
}
runningflag = True
state = 'normal'
flag_icp = False
DOWNSAMPLE_VOXEL_SIZE_M = 0.005
scene_rotation_g = None
model_pcd_g = None
scene_pcd_g = None


def on_press(key):
        global object_pose,runningflag,moving_speed
        global state, flag_icp
        global scene_rotation_g
        global model_pcd_g, scene_pcd_g

        def get_global_translation(local_translation):
            global object_pose
            global_translation = np.matmul(
                object_pose['pose'][:3, :3], np.asarray(local_translation).reshape((3, 1))
            )[:3, 0]
            return global_translation

        if state == 'normal':
                try:
                        if key.char=='d':
                                object_pose['pose'][:3, -1] += get_global_translation([0, -0.001 * moving_speed, 0])
                                object_pose['is_new'] = True
                        elif key.char == 'a':
                                object_pose['pose'][:3, -1] += get_global_translation([0, 0.001 * moving_speed, 0])
                                object_pose['is_new'] = True
                        elif key.char == 'w':
                                object_pose['pose'][:3, -1] += get_global_translation([0.001 * moving_speed, 0, 0])
                                object_pose['is_new'] = True
                        elif key.char == 's':
                                object_pose['pose'][:3, -1] += get_global_translation([-0.001 * moving_speed, 0, 0])
                                object_pose['is_new'] = True
                        elif key.char == 'e':
                                object_pose['pose'][:3, -1] += get_global_translation([0, 0, 0.001 * moving_speed])
                                object_pose['is_new'] = True
                        elif key.char == 'c':
                                object_pose['pose'][:3, -1] += get_global_translation([0, 0, -0.001 * moving_speed])
                                object_pose['is_new'] = True
                        elif key.char == 'j':
                                del_R = o3d.geometry.get_rotation_matrix_from_xyz([-np.deg2rad(moving_speed), 0, 0])
                                object_pose['pose'][:3, :3] = object_pose['pose'][:3, :3] @ del_R
                                object_pose['is_new'] = True
                        elif key.char == 'l':
                                del_R = o3d.geometry.get_rotation_matrix_from_xyz([np.deg2rad(moving_speed), 0, 0])
                                object_pose['pose'][:3, :3] = object_pose['pose'][:3, :3] @ del_R
                                object_pose['is_new'] = True
                        elif key.char == 'i':
                                del_R = o3d.geometry.get_rotation_matrix_from_yzx([np.deg2rad(moving_speed), 0, 0])
                                object_pose['pose'][:3, :3] = object_pose['pose'][:3, :3] @ del_R
                                object_pose['is_new'] = True
                        elif key.char == 'k':
                                del_R = o3d.geometry.get_rotation_matrix_from_yzx([-np.deg2rad(moving_speed), 0, 0])
                                object_pose['pose'][:3, :3] = object_pose['pose'][:3, :3] @ del_R
                                object_pose['is_new'] = True
                        elif key.char == 'u':
                                del_R = o3d.geometry.get_rotation_matrix_from_zyx([-np.deg2rad(moving_speed), 0, 0])
                                object_pose['pose'][:3, :3] = object_pose['pose'][:3, :3] @ del_R
                                object_pose['is_new'] = True
                        elif key.char == 'm':
                                del_R = o3d.geometry.get_rotation_matrix_from_zyx([np.deg2rad(moving_speed), 0, 0])
                                object_pose['pose'][:3, :3] = object_pose['pose'][:3, :3] @ del_R
                                object_pose['is_new'] = True
                        elif key.char == 'z':
                            # align object with scene OBB
                            if scene_rotation_g is not None:
                                object_pose['pose'][:3, :3] = scene_rotation_g.copy()
                                object_pose['is_new'] = True
                            else:
                                print('err: scene_rotation_g is None')
                        elif key.char == 'x':
                            # ICP align object and scene
                            print('running ICP')
                            icp_results = icp_align(model_pcd_g, scene_pcd_g, object_pose['pose'])
                            if icp_results['success']:
                                object_pose['pose'] = icp_results['pose'].copy()
                                object_pose['is_new'] = True
                                print('ICP succeeded')
                            else:
                                print('ICP failed')
                        elif key.char == ']':
                                moving_speed *= 5
                                object_pose['is_new'] = True
                        elif key.char == '[':
                                if moving_speed > 1:
                                        moving_speed /= 5
                                        object_pose['is_new'] = True
                except AttributeError:
                        if key == keyboard.Key.enter:
                                state = 'confirm'
                        if key == keyboard.Key.space:
                                flag_icp = True
        elif state == 'confirm':
                if key == keyboard.Key.enter:
                        runningflag = False
                        return False
                else:
                        state = 'normal'

def on_release(key):
        pass

def main():
        global runningflag,x,y,z,alpha,beta,gamma
        global moving_speed, flag_icp, num_frame
        global scene_rotation_g
        global model_pcd_g, scene_pcd_g

        font_size = 0.5
        font_thickness = 1
        font = cv2.FONT_HERSHEY_SIMPLEX
        font_color = (255,0,255)

        object_file_name_list_file = open(OBJECT_FILE_NAME_LIST_FILE_NAME,'r')
        lines = object_file_name_list_file.readlines()
        objectfilenamelist = []
        for line in lines:
                if not (line == '\n'):
                        objectfilenamelist.append(line.replace('\n','').replace('\r',''))
        print('log:loaded object file name list:',end='')
        print(objectfilenamelist)

        objectfilenamedict={}
        for i, obj in enumerate(objectfilenamelist):
                objectfilenamedict[obj]=i

        object_id_list_file = open(OBJECT_ID_LIST_FILE_NAME,'r')
        lines = object_id_list_file.readlines()
        objectidlist=[]

        for line in lines:
                if not (line == '\n'):
                        objectidlist.append(objectfilenamedict[line.strip('\n')])
        print('log:loaded object id list:',end='')
        print(objectidlist)

        print('log:loading camera parameters')
        if FLAGS.camera == 'realsense':
                cam = np.load(os.path.join(DATA_DIR, 'color_camK.npy'))
        elif FLAGS.camera == 'kinect':
                cam = np.array([631.54864502,0.0,638.43517329,0.,631.20751953,366.49904066,0.0,0.0,1.0]).reshape((3, 3))
        else:
                raise ValueError('Wrong type of camera')
        # cam = np.array([631.5486, 0.0, 638.435, 0.0, 631.21, 366.5, 0.0, 0.0, 1.0]).reshape((3, 3))
        print('log:camera parameter:\n'+str(cam))
        num_rgb_image = len(os.listdir(RGB_IMAGE_DIR))
        num_depth_image = len(os.listdir(DEPTH_IMAGE_DIR))
        assert num_depth_image == num_rgb_image
        # frameposevectorlist format
        # [frame1posevectorlist,frame2.....]
        # frameposevectorlist format
        # [object1 posevector,object2 posevector]
        # object pose vector format
        # [id,x,y,z,alpha,beta,gamma] angle in unit of degree
        # frameposevectorlist[num_frame][objectid] = [id,x,y,z,alpha,beta,gamma]
        frame_number = num_depth_image
        print('log:total frames number:%d' % frame_number)
        frameposevectorlist = getframeposevectorlist(objectidlist,IS_RESUME,frame_number,XML_DIR)
        models = []
        models_ply = []
        for i in range(len(objectidlist)):
                models.append(None)
                models_ply.append(None)

        # if os.path.exists(PLY_DIR):
        #       scene = o3d.io.read_point_cloud(PLY_DIR)
        #       scene = scene.voxel_down_sample(DOWNSAMPLE_VOXEL_SIZE_M)
        # else:
        #       print('log: construct full point cloud')
        #       scene = lego_icp(FLAGS.camera,ALL_DATA_DIR,CAMERA_POSE_NUMPY_FILE_NAME)
        #       scene = scene.voxel_down_sample(DOWNSAMPLE_VOXEL_SIZE_M)
        #       o3d.io.write_point_cloud(PLY_DIR, scene)


        print('log:scene loaded')
        cameraposes = np.load(CAMERA_POSE_NUMPY_FILE_NAME, allow_pickle=True).item()

        cv2.namedWindow('2D Vis', cv2.WINDOW_NORMAL | cv2.WINDOW_KEEPRATIO)

        while True:
                moving_speed = 5
                existed_object_file_name_list = []
                for objectid in objectidlist:
                        existed_object_file_name_list.append(objectfilenamelist[objectid])

                print('log:loading image '+os.path.join(RGB_IMAGE_DIR,'color_%d.png' % (num_frame+1)))
                image = cv2.imread(os.path.join(RGB_IMAGE_DIR,'color_%d.png' % (num_frame+1)))

                mask = cv2.imread(
                    os.path.join(RGB_IMAGE_DIR, '..', 'mask_rcnn_masks', 'color_%d.png' % (num_frame+1)),
                    cv2.IMREAD_ANYCOLOR
                )
                mask = np.stack((mask, mask, mask), axis=-1)
                mask[:, :, 0] = 0
                mask[:, :, 2] = 0
                image = cv2.addWeighted(
                    image, 1.0, mask, 0.95, 0
                )

                print('log:image loaded')

                textimage = copy.deepcopy(image)
                textimage = cv2.putText(textimage, 'Frame %d Input an intager to select ply file, 0 for exiting' % num_frame,(10, 30),font, font_size, font_color, font_thickness)
                textimage = cv2.putText(textimage, '0:exit',(10, 55),font, font_size, font_color, font_thickness)
                textimage = cv2.putText(textimage, '1:last image',(10, 80),font, font_size, font_color, font_thickness)
                textimage = cv2.putText(textimage, '2:next image',(10, 105),font, font_size, font_color, font_thickness)
                for i in range(len(existed_object_file_name_list)):
                        if i < 7:
                                textimage = cv2.putText(textimage, str(i+3)+':'+existed_object_file_name_list[i],(10, 55 + 25 * (i + 3)),font, font_size, font_color, font_thickness)
                        else:
                                textimage = cv2.putText(textimage, chr(ord('a') + i - 7)+':'+existed_object_file_name_list[i],(10, 55 + 25 * (i + 3)),font, font_size, font_color, font_thickness)
                cv2.imshow('2D Vis',textimage)

                getkey = cv2.waitKey(0)
                while getkey < ord('0') or getkey > ord('z') or (getkey < ord('a') and getkey > ord('9')):
                        getkey = cv2.waitKey(0)
                if getkey - ord('0') <= 9:
                        plyfilenumber = getkey - ord('0')
                else:
                        plyfilenumber = getkey - ord('a') + 10

                if plyfilenumber == 0:
                        break
                elif plyfilenumber == 1:
                        while True:
                            num_frame = (num_frame - 1) % frame_number
                            if (num_frame+1) in cameraposes:
                                break
                        print('log:select last image:')
                        continue
                elif plyfilenumber == 2:
                        while True:
                            num_frame = (num_frame + 1) % frame_number
                            if (num_frame+1) in cameraposes:
                                break
                        print('log:select next image:')
                        continue
                else:
                        plyfilenumber -= 3
                # the plyfilenumber is out of range
                if plyfilenumber >= len(existed_object_file_name_list):
                        continue

                # read the npy file for pose if exists
                pose = None
                object_pose_file = os.path.join(FLAGS.data_dir, 'object_poses.npy')
                if os.path.isfile(object_pose_file):
                    object_poses = np.load(object_pose_file, allow_pickle=True).item()
                    if existed_object_file_name_list[plyfilenumber] in object_poses:
                        pose = object_poses[existed_object_file_name_list[plyfilenumber]]
                if pose is None:
                    pose = np.eye(4)
                    pose[2, 3] = 0.4

                object_pose['pose'] = pose.copy()
                object_pose['is_new'] = True

                textimage2 = copy.deepcopy(image)
                textimage2 = cv2.putText(textimage2, 'loading model '+os.path.join(MODEL_DIR,existed_object_file_name_list[plyfilenumber]),(10, 30),font, font_size, font_color, font_thickness)
                cv2.imshow('2D Vis',textimage2)
                cv2.waitKey(5)
                print('log:loading model '+os.path.join(MODEL_DIR,existed_object_file_name_list[plyfilenumber]))
                if models[plyfilenumber] is None:
                        assert models_ply[plyfilenumber] is None
                        print(MODEL_DIR, existed_object_file_name_list[plyfilenumber])
                        models[plyfilenumber] = 1 # loadmodel(MODEL_DIR,existed_object_file_name_list[plyfilenumber])
                        # cachemodel(MODEL_DIR,existed_object_file_name_list[plyfilenumber],models[plyfilenumber])
                        downsample_name = existed_object_file_name_list[plyfilenumber].split('.ply')[0]+'_downsample.ply'
                        if os.path.exists(os.path.join(MODEL_DIR,downsample_name)):
                                model_path = os.path.join(MODEL_DIR,downsample_name)
                                models_ply[plyfilenumber] = o3d.io.read_triangle_mesh(
                                    model_path, enable_post_processing=True
                                )
                        else:
                                print('log:downsampling model')
                                model_path = os.path.join(MODEL_DIR,existed_object_file_name_list[plyfilenumber], 'visual.ply')
                                model_obj_path = os.path.join(MODEL_DIR,existed_object_file_name_list[plyfilenumber], 'textured.obj')
                                models_ply[plyfilenumber] = o3d.io.read_triangle_mesh(model_obj_path, enable_post_processing=True)
                                # models_ply[plyfilenumber].voxel_down_sample(DOWNSAMPLE_VOXEL_SIZE_M)
                        print('log:model loaded')
                else:
                        print('using cached model')
                # trans_all = np.load('cam_poses_new.npy')

                model_pcd_g = models_ply[plyfilenumber].sample_points_uniformly(
                        number_of_points=OBJECT_MESH_SAMPLE_POINTS,
                        use_triangle_normal=True
                )

                scene_ply = o3d.io.read_triangle_mesh(
                        os.path.join(ALL_DATA_DIR, 'fastfusion_reconstruction.ply')
                )
                if scene_ply.has_triangles():
                    scene_pcd_g = scene_ply.sample_points_uniformly(
                            number_of_points=SCENE_MESH_SAMPLE_POINTS,
                            use_triangle_normal=True
                    )
                else:
                    scene_pcd_g = o3d.geometry.PointCloud(scene_ply.vertices)
                    if scene_ply.has_vertex_colors():
                        scene_pcd_g.colors = scene_ply.vertex_colors
                    else:
                        scene_pcd_g.paint_uniform_color([0.706, 0.706, 0.706])
                scene_obb = scene_ply.get_oriented_bounding_box()

                # TODO: multiple rotations that aligns object to scene possible
                scene_rotation_g = get_scene_rotation(scene_obb, cameraposes)
                print('log:scene loaded')

                origin_axes = o3d.geometry.TriangleMesh.create_coordinate_frame(size=0.1, origin=[0, 0, 0])
                scene_axes = o3d.geometry.TriangleMesh \
                                        .create_coordinate_frame(size=0.5, origin=[0, 0, 0]) \
                                        .rotate(scene_rotation_g) # .translate(scene_obb.get_center())

                model_global = copy.deepcopy(models_ply[plyfilenumber]).transform(pose)
                obj_axes = o3d.geometry.TriangleMesh.create_coordinate_frame(size=0.1, origin=[0, 0, 0]).transform(pose)

                o3d_vis = o3d.visualization.Visualizer()
                o3d_vis.create_window(window_name='3D Vis')
                o3d_vis.add_geometry(model_global)

                # if scene_ply.has_triangles():
                #     o3d_vis.add_geometry(scene_ply)
                # else:
                #     o3d_vis.add_geometry(scene_pcd_g)

                # visualize point cloud instead of mesh
                scene_pcd = o3d.geometry.PointCloud(scene_ply.vertices)
                scene_pcd.colors = scene_ply.vertex_colors
                o3d_vis.add_geometry(scene_pcd)

                o3d_vis.add_geometry(obj_axes)
                # o3d_vis.add_geometry(scene_axes)
                # o3d_vis.add_geometry(origin_axes)

                runningflag = True
                # begin monitoring th222139e keyboard event in non-blocking form
                listener = keyboard.Listener(on_press=on_press,on_release=on_release)
                listener.start()

                while runningflag:
                        if flag_icp:
                                # scene_transform = scene.transform(np.linalg.inv(cameraposes[num_frame]))

                                scene_transform = copy.deepcopy(scene)
                                scene_transform.transform(np.linalg.inv(cameraposes[num_frame]))
                                # o3d.io.write_point_cloud('./transform.ply', scene_transform, write_ascii=True)
                                x, y, z, alpha, beta, gamma = refine_6d_pose(os.path.join(RGB_IMAGE_DIR,'color_%d.png' % (num_frame+1)),\
                        os.path.join(DEPTH_IMAGE_DIR,'color_%d.png' % (num_frame+1)),scene_transform, models_ply[plyfilenumber], cam, x, y, z, alpha, beta, gamma)
                                flag_icp = False

                        if object_pose['is_new']:
                            object_pose['is_new'] = False
                            previous_pose = pose.copy()
                            pose = object_pose['pose'].copy()
                            pose_change = np.matmul(pose, np.linalg.inv(previous_pose))

                            rendered_image=draw_model(image, pose, cam, cameraposes[num_frame+1], model_path)
                            # rendered_image = copy.deepcopy(image)

                            x, y, z = pose[:3, -1].tolist()
                            alpha, beta, gamma = np.rad2deg(mat2euler(pose[:3, :3])).tolist()
                            rendered_image = cv2.putText(rendered_image, 'x:%.3f y:%.3f z:%.3f alpha:%d beta:%d gamma:%d moving speed:%d' % (x,y,z,alpha,beta,gamma,moving_speed),\
                            (20, image.shape[0] - 10),font, font_size, font_color, font_thickness)
                            global state

                            if state == 'confirm':
                                    rendered_image = cv2.putText(rendered_image, 'Confirm finishing this object?',(20, 25),font, font_size, font_color, font_thickness)
                                    rendered_image = cv2.putText(rendered_image, 'Press "enter" to confirm, others to resume',(20, 50),font, font_size, font_color, font_thickness)
                            cv2.imshow('2D Vis', rendered_image)

                            model_global.transform(pose_change)
                            o3d_vis.update_geometry(model_global)

                            obj_axes.transform(pose_change)
                            o3d_vis.update_geometry(obj_axes)

                        o3d_vis.poll_events()
                        o3d_vis.update_renderer()

                        cv2.waitKey(5)

                print('log:saving object poses')
                np.save(
                    os.path.join(FLAGS.data_dir, 'object_poses.npy'),
                    {
                        existed_object_file_name_list[plyfilenumber]: object_pose['pose']
                    }
                )
                o3d_vis.destroy_window()
                cv2.destroyAllWindows()
                tear_down_renderer()

                # frameposevectorlist[num_frame][plyfilenumber]=[objectidlist[plyfilenumber],x,y,z,alpha,beta,gamma]

                # if IS_SAVE_XML:
                #         if not os.path.exists(XML_DIR):
                #                 print('log: create directory '+XML_DIR)
                #                 os.mkdir(XML_DIR)
                #         if IS_EXTEND_FRAMES:
                #                 mainFrameTransformer = frameTransformer(cameraPosesNpyFile=CAMERA_POSE_NUMPY_FILE_NAME,
                #                 baseNumFrame = num_frame,
                #                 framePoseVectorList=frameposevectorlist,
                #                 objnamelist = objectfilenamelist,
                #                 objidlist = objectidlist,
                #                 cam=cam,
                #                 models_ply = models_ply,
                #                 existed_object_file_name_list=existed_object_file_name_list,
                #                 rgb_image_dir = RGB_IMAGE_DIR,
                #                 depth_image_dir = DEPTH_IMAGE_DIR,
                #                 is_icp=IS_ICP)
                #                 mainFrameTransformer.transform()

                #                 for n_frame in range(frame_number):
                #                         mainxmlWriter = xmlWriter()
                #                         mainxmlWriter.objectlistfromposevectorlist(posevectorlist=frameposevectorlist[n_frame],
                #                         objdir=MODEL_DIR,
                #                         objnamelist=objectfilenamelist,
                #                         objidlist=objectidlist)
                #                         mainxmlWriter.writexml(xmlfilename = os.path.join(XML_DIR,'%04d.xml' % (n_frame+1)))
                #         else:
                #                 mainxmlWriter = xmlWriter()
                #                 mainxmlWriter.objectlistfromposevectorlist(posevectorlist=frameposevectorlist[num_frame],
                #                 objdir=MODEL_DIR,
                #                 objnamelist=objectfilenamelist,
                #                 objidlist=objectidlist)
                #                 mainxmlWriter.writexml(xmlfilename= os.path.join(XML_DIR,'%04d.xml' % (num_frame+1)))


def icp_align(
        object_pcd, scene_pcd, initial_pose,
        alignment_threshold=ICP_ALIGNMENT_THRESHOLD, use_normal=True
):
    if (object_pcd is None) or (scene_pcd is None) or (initial_pose is None):
        return {'success': False}

    try:
        registration = o3d.registration
    except AttributeError:
        registration = o3d.pipelines.registration

    if use_normal and scene_pcd.has_normals():
        # TODO: load color in object model and do colored ICP
        # registration.TransformationEstimationForColoredICP(),
        kwargs = {
            'estimation_method': \
                registration.TransformationEstimationPointToPlane()
        }
    else:
        kwargs = {
            'estimation_method': \
                registration.TransformationEstimationPointToPoint()
        }

    reg_p2l = registration.registration_icp(
        object_pcd, scene_pcd, alignment_threshold, initial_pose,
        criteria=registration.ICPConvergenceCriteria(
            relative_fitness=1e-6,
            relative_rmse=1e-6,
            max_iteration=1000
        ),
        **kwargs
    )

    return {
        'success': True,
        'pose': reg_p2l.transformation,
        'fitness': reg_p2l.fitness,
        'inlier_rmse': reg_p2l.inlier_rmse,
        'correspondences': np.asarray(reg_p2l.correspondence_set).shape[0]
    }


def get_scene_rotation(scene_obb, camera_poses):
    scene_rotation = correct_rotation(scene_obb.R)

    flip_rotation = 0
    for T in camera_poses.values():
        # camera Y-axis and scene Z-axis should be close to 180 degrees
        if np.dot(T[:3, 1], scene_rotation[:3, 2]) > 0:
            flip_rotation += 1

    if flip_rotation > 0:
        scene_rotation = scene_rotation \
                @ o3d.geometry.get_rotation_matrix_from_yzx([np.pi, 0, 0])

    return scene_rotation


if __name__ == '__main__':
        main()
