from trans3d import get_mat, get_pose
import open3d as o3d
import numpy as np
import cv2
import os
import copy

MAX_DEPTH_M = 0.7
MIN_DEPTH_M = 0.1

WIDTH = 1280
HEIGHT = 720

def rgbd_with_mask_to_pointcloud(color_image_name, depth_image_name, mask_name, width, height, camera_matrix,object_id):
    color = o3d.io.read_image(color_image_name)
    depth = cv2.imread(depth_image_name, cv2.IMREAD_UNCHANGED).astype(np.float32)
    mask = np.load(mask_name)
    depth /= 1000.0
    depth[depth < MIN_DEPTH_M] = 0
    depth[depth > MAX_DEPTH_M] = 0
    print(depth.dtype)
    mask_01 = (mask == object_id).astype(int)
    print(mask_01)

    depth = (depth * mask_01).astype(np.float32)
    
    rgbd_image = o3d.geometry.RGBDImage()
    rgbd_image.color = color
    rgbd_image.depth = o3d.geometry.Image(depth)

    intrinsic = o3d.camera.PinholeCameraIntrinsic()
    intrinsic.set_intrinsics(
        width, height, camera_matrix[0, 0], camera_matrix[1, 1], camera_matrix[0, 2], camera_matrix[1, 2])
    pcd = o3d.geometry.PointCloud()
    pcd = o3d.geometry.PointCloud.create_from_rgbd_image(rgbd_image, intrinsic, np.eye(4))
    # pcd = o3d.geometry.create_point_cloud_from_rgbd_image(rgbd_image, intrinsic, np.eye(4))
    return pcd


cam_rs = np.array([927.17, 0.0, 651.32, 0.0, 927.37, 349.62, 0.0, 0.0, 1.0]).reshape((3, 3))
cam_kn = np.array([631.54864502,0.0,638.43517329,0.,631.20751953,366.49904066,0.0,0.0,1.0]).reshape((3, 3))

rgb_path = 'scene_33/data/rgb/0000.png'
depth_path ='scene_33/data/depth/0000.png'
mask_path = 'scene_33/data/masks/0000.npy'

mask = np.load(mask_path)
pcd = rgbd_with_mask_to_pointcloud(rgb_path,depth_path,mask_path,WIDTH,HEIGHT,cam_rs,22)

vis = o3d.visualization.Visualizer()
vis.create_window(width=1280, height=720)
vis.add_geometry(pcd)
vis.run()