import shutil
import os
import tarfile
from xmlhandler import xmlReader
def generate_object_id_list(scene_directory, scene_name):
    xml_file_path=os.path.join(scene_directory,scene_name,'data','annotations','0000.xml')
    main_xml_reader = xmlReader(xml_file_path)
    pose_vector_list = main_xml_reader.getposevectorlist()
    with open(os.path.join(scene_name,'object_id_list.txt'),'w') as f:
        for pose_vector in pose_vector_list:
            f.write(str(pose_vector[0])+'\n')

def unpack(scene_name,tar_file_directory = '.',scene_directory = '.',force_move=True):
    """
    Input: the scene name(scene_31 for example)
    tar_file_directory: where are the tar files
    scene_directory: where are the scene folders

    Output: True for success,False For Fail

    Function: Unpack the annotations in the tar file into the scene folder
    """
    abs_scene_directory = os.path.join(scene_directory,scene_name)
    # check if there is already annotations in this folder
    if os.path.exists(os.path.join(abs_scene_directory,'data','annotations')):
        print('log:Warning: %s already exists' % os.path.join(abs_scene_directory,'data','annotations'))
        if not force_move:
            raise ValueError('Directory %s already exists' % os.path.join(abs_scene_directory,'data','annotations'))
        shutil.rmtree(os.path.join(abs_scene_directory,'data','annotations'))

    if os.path.exists(os.path.join(abs_scene_directory,'data_kinect','annotations')):
        print('log:Warning: %s already exists' % os.path.join(abs_scene_directory,'data_kinect','annotations'))
        if not force_move:
            raise ValueError('Directory %s already exists' % os.path.join(abs_scene_directory,'data_kinect','annotations'))
        shutil.rmtree(os.path.join(abs_scene_directory,'data_kinect','annotations'))

    tar_files = os.listdir(tar_file_directory)
    tar_file_name = None
    for tar_file in tar_files:
        # ignore all any file whose postfix is not .tar
        if not tar_file[-4:] == '.tar':
            continue
        try:
            name,student_id,student_name = tar_file.split('-')
        except:
            # deal with tar file 
            print('log:Unrecognized tar file:%s' % tar_file)
            continue
        student_name = student_name[:-4]
        print(name, '   ', scene_name)
        if name  == scene_name:
            tar_file_name = tar_file
            print('log:Found tar file, scene name:%s, student id:%s, student name:%s'% (name,student_id,student_name))
            break
    assert tar_file_name is not None
    with tarfile.open(os.path.join(tar_file_directory,tar_file_name)) as tar:
        temp_int = 0
        # find a temp dir name that does not exist
        while(os.path.exists('temp_dir_%d' % temp_int)):
            temp_int += 1
        temp_dir = 'temp_dir_%d' % temp_int
        os.mkdir(temp_dir)
        print('log:Extracting Files')
        tar.extractall(temp_dir)
    print('log:Coping Files')
    shutil.copytree(os.path.join(temp_dir,'rs','annotations'),os.path.join(abs_scene_directory,'data','annotations'))
    shutil.copytree(os.path.join(temp_dir,'kn','annotations'),os.path.join(abs_scene_directory,'data_kinect','annotations'))
    print('log:Removing Temporary Directory')
    shutil.rmtree(temp_dir)

    generate_object_id_list(scene_directory,scene_name)
    print('log:generated object_id_list.txt')
    return True