#!/usr/bin/env python
import logging
import datetime
import argparse
from pathlib import Path
import subprocess

from cv2 import resize

def init_logger():
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)

    fh = logging.FileHandler('../dataset/mask_based_pose_estimation_all_{}.txt'.format(str(datetime.datetime.now())))
    fh.setFormatter(logging.Formatter('%(asctime)-15s -%(levelname)s - %(message)s'))
    fh.setLevel(logging.INFO)

    logger.addHandler(fh)

    return logger

logger = init_logger()


def pose_estmimation(dataset_root, scene_name):
    cmd = ['python', 'mask_based_pose_estimation.py', '--dataset-root', str(dataset_root), '--scene-name', scene_name, '--resize-factor', '1']
    logger.info('running: ' + ' '.join(cmd))
    ret = subprocess.call(cmd)
    if ret != 0:
        logger.error('Problem with' + ' '.join(cmd))

def visualize_poses(dataset_root, scene_name, resize_factor):
    cmd = ['python', 'visualize_poses.py', '--dataset-root', str(dataset_root), '--scene-name', scene_name, '--resize-factor', str(resize_factor)]
    logger.info('running: ' + ' '.join(cmd))
    ret = subprocess.call(cmd)
    if ret != 0:
        logger.error('Problem with' + ' '.join(cmd))

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--dataset-root', type=Path
    )

    args = parser.parse_args()
    dataset_root = args.dataset_root

    scenes_root = dataset_root/'scenes'
    scene_list_path = dataset_root/'scene_name_list.txt'
    with scene_list_path.open('r') as scene_list_file:
        scene_list = scene_list_file.read().splitlines()
        scene_list_file.close()
    scene_list.sort()

    annotated_scenes_path = dataset_root/'annotated_scenes.txt'
    with annotated_scenes_path.open('r') as annotated_scenes_file:
        annotated_scenes_list = annotated_scenes_file.read().splitlines()
        annotated_scenes_file.close()

    logger.info('starting the batch mask based pose estimation')
    for scene_name in scene_list:
        if(scenes_root/scene_name).exists() and (scenes_root/scene_name/'object_poses.npy').exists() and \
            len(list((scenes_root/scene_name).glob('seg_masks*')))==0:
            pose_estmimation(dataset_root, scene_name)
            visualize_poses(dataset_root, scene_name, 1)











