#!/usr/bin/env bash

DATASET_ROOT=$1
SCENE_NAME=$2
RESIZE_FACTOR=1

# undistort images
python undistort_images.py \
    --dataset-root "$DATASET_ROOT" --scene-name "$SCENE_NAME" \
    --resize-factor "$RESIZE_FACTOR"
retVal=$?
if [ $retVal -ne 0 ]; then
    echo "Error in undistort"
    exit $retVal
fi

ulimit -Sn 10000

# render objects at random pose
python renderer.py \
    --dataset-root "$DATASET_ROOT" --scene-name "$SCENE_NAME" \
    --resize-factor "$RESIZE_FACTOR" --num-real-images 120
retVal=$?
if [ $retVal -ne 0 ]; then
    echo "Error in renderer"
    exit $retVal
fi

# Run COLMAP sfm
python sfm.py \
    --images "$DATASET_ROOT/scenes/$SCENE_NAME/all_images" \
    --outputs "$DATASET_ROOT/scenes/$SCENE_NAME/hloc"
retVal=$?
if [ $retVal -ne 0 ]; then
    echo "Error in sfm"
    exit $retVal
fi

# convert SFM model to TXT format
mkdir -p "$DATASET_ROOT/scenes/$SCENE_NAME/colmap_output_all_images_2"
colmap model_converter \
    --input_path "$DATASET_ROOT/scenes/$SCENE_NAME/hloc/sfm_superpoint+superglue/models/0" \
    --output_path "$DATASET_ROOT/scenes/$SCENE_NAME/colmap_output_all_images_2" \
    --output_type TXT
retVal=$?
if [ $retVal -ne 0 ]; then
    echo "Error in model_converter"
    exit $retVal
fi

# Run pose estimation
python pose_estimation.py \
    --dataset-root "$DATASET_ROOT" --scene-name "$SCENE_NAME" \
    --resize-factor "$RESIZE_FACTOR"
retVal=$?
if [ $retVal -ne 0 ]; then
    echo "Error in pose_estimation"
    exit $retVal
fi
