#!/usr/bin/env python
import sys
sys.path.append('dataset_toolkit')

from toc_dataset_toolkit import TOC_Dataset
from toc_dataset_toolkit.utils.logging import set_log_level

import argparse
import os
from pathlib import Path
import subprocess
import logging
import datetime


def main(dataset_root):
    set_log_level('WARNING')
    dataset = TOC_Dataset(root=dataset_root)
    scene_name_list = dataset.load_scene_name_list()
    scene_name_list.sort()

    project_path = Path(os.path.realpath(__file__)).parents[0]
    dataset_root = Path(os.path.abspath(str(dataset_root)))

    scenes_root = dataset_root/'scenes'
    finished_scenes_file_path = dataset_root/(str(datetime.datetime.now())+'.txt')

    with finished_scenes_file_path.open('w') as finished_scenes_file:
        for scene_name in scene_name_list:
            if (scenes_root/scene_name).exists() and not (scenes_root/scene_name/'colmap_output_all_images_2').exists():
                finished_scenes_file.write(str(datetime.datetime.now())+': ' + scene_name + ' annotation started.\n')
                finished_scenes_file.flush()
                run_cmd([
                    'bash', str(project_path / 'run_full_sift.sh'),
                    str(dataset_root), str(scene_name)
                ])
                finished_scenes_file.write(str(datetime.datetime.now())+': ' + scene_name + ' annotation finished.\n')


def run_cmd(cmd):
    logging.info('running: ' + ' '.join(cmd))
    ret = subprocess.call(cmd)
    if ret != 0:
        logging.error('Problem with ' + ' '.join(cmd))


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--dataset-root', type=Path,
        default='/root/3d_dataset'
    )
    args = parser.parse_args()
    main(**args.__dict__)
