#!/usr/bin/env python
import logging
import datetime
import argparse
from pathlib import Path
import subprocess

def init_logger():
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)

    fh = logging.FileHandler('../dataset/run_textureless_all_{}.txt'.format(str(datetime.datetime.now())))
    fh.setFormatter(logging.Formatter('%(asctime)-15s -%(levelname)s - %(message)s'))
    fh.setLevel(logging.INFO)

    logger.addHandler(fh)

    return logger

logger = init_logger()


def run_textureless(dataset_root, scene_name):
    cmd = ['bash', 'run_textureless.sh', str(dataset_root), scene_name]
    logger.info('running: ' + ' '.join(cmd))
    ret = subprocess.call(cmd)
    if ret != 0:
        logger.error('Problem with' + ' '.join(cmd))



if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--dataset-root', type=Path
    )

    args = parser.parse_args()
    dataset_root = args.dataset_root

    scenes_root = dataset_root/'scenes'
    scene_list_path = dataset_root/'scene_name_list.txt'
    with scene_list_path.open('r') as scene_list_file:
        scene_list = scene_list_file.read().splitlines()
        scene_list_file.close()
    scene_list.sort()

    logger.info('starting the batch run textureless')
    for scene_name in scene_list:
        if(scenes_root/scene_name).exists() and (not (scenes_root/scene_name/'fastfusion_reconstruction.ply').exists() \
            or not (scenes_root/scene_name/'CameraTrajectory.txt').exists() or not (scenes_root/scene_name/'KeyFrameTrajectory.txt').exists()):
            run_textureless(dataset_root, scene_name)









