#!/usr/bin/env python


import argparse
from pathlib import Path
import cv2
import numpy as np
from tqdm import tqdm


def main(dataset_root, scene_name, resize_factor):
    path = dataset_root / 'scenes' / scene_name
    distorted_path = path / 'distorted_images'
    undistorted_path = path / 'rgb_undistort'
    intrinsics_path = path / 'color_camK.npy'
    distortion_coeff_path = path / 'color_cam_distortion_coeffs.npy'

    undistorted_path.mkdir(exist_ok=True, parents=True)

    K = np.load(intrinsics_path).reshape((3, 3))
    distortion_coeff = np.load(distortion_coeff_path).reshape((-1,))
    assert(K.size == 9)

    for filepath in tqdm(list(distorted_path.iterdir())):
        if not filepath.name.startswith('color_'):
            continue

        distorted_img = cv2.imread(str(filepath))
        undistorted_img = cv2.undistort(distorted_img, K, distortion_coeff)
        undistorted_img = cv2.resize(undistorted_img, (
            int(undistorted_img.shape[1]/resize_factor),
            int(undistorted_img.shape[0]/resize_factor)
        ))

        cv2.imwrite(str(undistorted_path / filepath.name), undistorted_img)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()

    parser.add_argument(
        '--dataset-root', type=Path,
        default='/root/3d_dataset'
    )
    parser.add_argument(
        '--scene-name', type=str,
        default='scene2'
    )
    parser.add_argument('--resize-factor', type=int, default=1)

    args = parser.parse_args()
    main(**args.__dict__)
